# SPDX-License-Identifier: Apache-2.0.
# Copyright (c) 2024 - 2025 Waldiez and contributors.
"""Version information for waldiez.

This file is automatically generated by Hatchling.
Do not edit this file directly.
"""
__version__ = VERSION = "0.6.3"
